require 'httpclient'
require 'fileutils'
require 'securerandom'

# DBバックアップサーバのテストをするスクリプト
# 使用例:
#   ruby dbtest.rb <vagrant | test | demo | stg | prd > <wal | base | split | all> <cert_dir>
CONF = {
  urls: {
    vagrant: "https://vagrant-ginbee:9500/",
    demo: "https://dbbackup1.orca-ng.org/",
    test: "https://dbbackup1-test.orca.orcamo.jp/",
    stg: "https://dbbackup1-stg.orca.orcamo.jp/",
    prd: "https://dbbackup1.orca.orcamo.jp/",
  },
}

@host = CONF[:urls][ARGV[0].to_sym]
type = ARGV[1]
cert_dir = ARGV[2]
cert_file = cert_dir.split("/").last
cert_key = File.join(cert_dir, "#{cert_file}.pem")
cert = File.join(cert_dir, "#{cert_file}.crt")
cacert = File.join(cert_dir, "ca.crt")
@upfile = "/tmp/bktest.tmp"

@client = HTTPClient.new
@client.ssl_config.set_client_cert_file(cert, cert_key)
@client.ssl_config.load_trust_ca
@client.ssl_config.set_trust_ca(cacert)
# @client.debug_dev=STDOUT

def up(path, ok=[], headers={}, file=@upfile)
  puts "  --- #{path}(#{file})"
  boundary = SecureRandom.hex[0..5]
  res = nil
  File.open(file) do |io|
    postdata = {
      "file" => io,
    }
    res = @client.post(
      File.join(@host, path), postdata,
      {
        "content-type" => "multipart/form-data; boundary=#{boundary}",
        "HTTP-X-Backup-SHA1" => Digest::SHA1.file(file).hexdigest,
      }.merge(headers))
  end
  if ok[0] != res.status
    p [:faild, ok[0], res.status, res.body]
    p res.inspect
    exit 1
  end
  if !ok[1].match(res.body)
    p [:faild, ok[1], res.status, res.body]
    p res.inspect
    exit 1
  end
  p [:success, res.status, res.body]
end

# WALファイルのアップロードテスト
def test_wal
  puts "=== test_wal ==="
  # SHA1が合致する
  up("/backup/wal", [200, /Success/])
  # SHA1が合致しない
  up("/backup/wal", [400, /SHA1 checksum unmatched/], {"HTTP-X-Backup-SHA1" => "invalid"})
end

# BASEファイルのアップロードテスト
def test_base
  puts "=== test_base ==="
  # SHA1が合致する
  up("/backup/base", [200, /Success/])
  # SHA1が合致しない
  up("/backup/wal", [400, /SHA1 checksum unmatched/], {"HTTP-X-Backup-SHA1" => "invalid"})
end

# 分割ファイルのアップロードテスト
def test_split
  puts "=== test_split ==="
  system("split -b 1 -d -a 8 /tmp/bktest.tmp /tmp/bktest.tmp.")
  files = Dir.glob("/tmp/bktest.tmp.*").sort
  h = {
    "http-x-backup-concat-sha1" => Digest::SHA1.file(@upfile).hexdigest,
    "http-x-backup-split-index" => 0,
    "http-x-backup-split-max" => (files.size - 1),
  }
  # 結合後のSHA1が合致する
  files.each_with_index{|f, i|
    up("/backup/split_base", [200, /Success/], h.merge("http-x-backup-split-index" => i), f)
  }

  # 結合後のSHA1が合致しない
  h["http-x-backup-concat-sha1"] = "invalid"
  files[0..-2].each_with_index{|f, i|
    up("/backup/split_base", [200, /Success/], h.merge("http-x-backup-split-index" => i), f)
  }
  up("/backup/split_base", [400, /Concat file SHA1/], h.merge("http-x-backup-split-index" => files.size - 1), files.last)
end

# 全ケースを実行
def test_all
  test_wal
  test_base
  test_split
end

FileUtils.touch("/tmp/bktest.tmp")
File.write("/tmp/bktest.tmp", "0123456789")
begin
  send("test_#{type}")
ensure
  FileUtils.rm_f("/tmp/bktest.tmp")
  FileUtils.rm_f("/tmp/bktest.tmp.*")
end