require "fileutils"
require 'httpclient'
require 'digest/sha1'
require 'securerandom'
require 'open3'
require 'logger'
require 'ltsv'
require 'date'
require 'time'
require_relative "dbbackup/version"
require_relative "dbbackup/uploader"
require_relative "dbbackup/base_uploader"
require_relative "dbbackup/split_base_uploader"
require_relative "dbbackup/wal_uploader"
require_relative "dbbackup/mis_uploader"
require_relative "dbbackup/downloader"

module Ginbee
  module Dbbackup
    def self.run
      try_lock do
        suploader = SplitBaseUploader.new('/backup/split_base', conf['cert'], conf['cert_key'], conf['cacert'])
        wuploader = WALUploader.new('/backup/wal', conf['cert'], conf['cert_key'], conf['cacert'])
        muploader = MisUploader.new('/backup/mis', conf['cert'], conf['cert_key'], conf['cacert'])
        if suploader.need_backup? || Ginbee::Dbbackup.conf['force_fullbackup'] == true
          # フルバックアップを実行
          suploader.run
          # WALファイル情報をクリア
          wuploader.clear
        else
          loop{
            # WALログのアップロード
            break if not wuploader.run
          }
        end
        if muploader.need_upload?
          # レセプト日次分データの出力とアップロード(統合ストレージへの格納用)
          muploader.run
        end
      end
    rescue => ex
      logger.error("dbbackup error: #{ex.inspect}")
      raise ex
    end

    def self.logger
      @@logger ||= Logger.new(conf['logger_file'] || STDOUT, 'weekly')
      return @@logger
    end

    def self.logger=(l)
      @@logger = l
    end

    def self.conf
      @@conf ||= {}
    end

    # 指定したファイルを作りflockをかける
    # ロックが取れなかった場合はなにもしない
    def self.try_lock(&block)
      file = File.join(conf['work_dir'], "uploader.lock")
      FileUtils.mkdir_p(File.dirname(file))
      File.open(file, "w") do |f|
        if f.flock(File::LOCK_EX | File::LOCK_NB)
          begin
            block.call
          ensure
            f.flock(File::LOCK_UN)
          end
        else
          logger.warn("dbbackup warning: lock acquisition failed.(dbbackup-client already running)")
        end
      end
    end
  end
end
