# encoding: utf-8
require 'json'
require 'uri'

module Ginbee
  module Dbbackup
    # BASEバックアップを送信する
    class BaseUploader < Uploader
      INTERVAL_DAYS = 7

      # BASEバックアップが必要なタイミングか？
      def need_backup?
        # BASEバックアップをまだ実施していない(初回)
        if state[:time].nil?
          return true
        # BASEバックアップに失敗（再送）
        elsif state[:state] == "FAILED"
          return true
        else
          t = Time.parse(state[:time]) + (1 * 60 * 60 * 24 * INTERVAL_DAYS)
          next_time = Time.new(t.year, t.month, t.day, Dbbackup.conf['base_backup_hour'].to_i, Dbbackup.conf['base_backup_min'].to_i)
          return time_now > next_time
        end
      end

      private
      def create_backup_file
        check_server_config
        backup_file = base_backup_file()
        FileUtils.rm_f(backup_file)

        cmd = "pg_basebackup -D #{base_backup_dir} -Ft -z -R -X fetch"
        o,e,s = Open3.capture3(cmd)
        Dbbackup.logger.debug("executed pg_basebackup: cmd=<#{cmd}>,out=<#{o}>,err=<#{e}>")
        if not s.success?
          raise "pg_basebackup command error: process_status=<#{s.inspect},stdout=<#{o}>,stderr=<#{e}>"
        end
        return backup_file
      end

      def state_file
        return File.join(Dbbackup.conf['work_dir'], "base_upload.state")
      end

      def base_backup_dir
        return File.join(Dbbackup.conf['work_dir'], "ginbee_base_backup")
      end

      def base_backup_file
        return File.join(base_backup_dir, "base.tar.gz")
      end

      # テストでスタブ化するため定義
      def time_now
        return Time.now
      end
    end
  end
end
