require 'json'

module Ginbee
  module Dbbackup
    class Client
      def initialize(cert, cert_key, cacert)
        @client = HTTPClient.new(:agent_name => "ginbee-dbbackup-#{VERSION}")
        @client.ssl_config.set_client_cert_file(cert, cert_key)
        @client.ssl_config.load_trust_ca
        @client.ssl_config.set_trust_ca(cacert)
        @client.connect_timeout = Dbbackup.conf['send_timeout'] || 60 * 60 # 1h
        @client.send_timeout = Dbbackup.conf['send_timeout'] || 60 * 60    # 1h
      end

      def host
        host = Dbbackup.conf['host']
        if host.nil?
          check_server_config
          return Dbbackup.conf['host']
        end
        return host
      end

      private

      # バックアップ先のサーバ情報を確認
      def check_server_config(path = "/dbbackup_service/api/server")
        res = @client.get(File.join(Dbbackup.conf['sms_host'], path),
                          {dbbackup_api_key: Dbbackup.conf['dbbackup_api_key']},
                          {'Content-Type' => 'application/json'})
        if HTTP::Status.successful?(res.status)
          server = JSON.parse(res.content)
          raise "sendable backup server is not found" if server.empty?
          uri = URI::Generic.build(scheme: server["scheme"] || 'https', host: server["host"])
          uri.port = server["port"].to_i if server["port"] && server["port"].to_i != 0
          new_host = uri.normalize.to_s

          if Dbbackup.conf['host'].nil? || URI.parse(Dbbackup.conf['host']) != URI.parse(new_host)
            update_server_config(new_host)
          end
        else
          raise "check server config failed: http_response=<#{res.body}>"
        end
      end

      # 接続先の更新があれば設定ファイルを更新する
      def update_server_config(new_host)
        Dbbackup.logger.info("detected backup server changes: old=<#{Dbbackup.conf['host']}> -> new=<#{new_host}>")
        Dbbackup.conf['host'] = new_host
        File.open(File.join(Dbbackup.conf['work_dir'], "config.yml"), "a+") {|f|
          f.puts "# -- #{Time.now.iso8601} -- added by dbbackup-client. Do not change following line  -- "
          f.puts "host: #{new_host}"
        }
      end
    end
  end
end
