require 'tempfile'
require_relative 'client'

module Ginbee
  module Dbbackup
    class Downloader < Client
      def initialize
        super(Dbbackup.conf["cert"], Dbbackup.conf["cert_key"], Dbbackup.conf["cacert"])
      end

      def run
        curl_opt = curl_options
        Tempfile.create("dbbackup_download_list.json") do |f|
          f.close
          cmd("curl #{curl_opt} #{File.join(host, "download/list.json")} -o #{f.path}")
          raw_json = File.read(f.path)
          begin
            json = JSON.load(raw_json)
            json.each do |file|
              cmd("curl #{curl_opt} #{File.join(host, file["path"])} -o #{file["name"]}")
            end
          rescue JSON::ParserError => e
            raise "curl command error: invalid list.json format. message=<#{e.message}> response=<#{raw_json}>"
          end
        end
      end

      private

      def cmd(c)
        o,e,s = Open3.capture3(c)
        Dbbackup.logger.debug("executed curl: cmd=<#{c}>,out=<#{o}>,err=<#{e}>")
        if not s.success?
          raise "curl command error: process_status=<#{s.inspect},stdout=<#{o}>,stderr=<#{e}>"
        end
      end

      def curl_options
        curl_opt = [
          "--cacert #{Dbbackup.conf["cacert"]}",
          "--key #{Dbbackup.conf["cert_key"]}",
          "--cert #{Dbbackup.conf["cert"]}",
        ].join(" ")
        if Dbbackup.conf["dbbackup_api_key"]
          curl_opt << " -H 'HTTP-X-Dbbackup-Api-Key:#{Dbbackup.conf["dbbackup_api_key"]}'"
        end
        return curl_opt
      end
    end
  end
end
