require_relative 'client'

module Ginbee
  module Dbbackup
    class Uploader < Client
      attr_reader :state

      def initialize(url_path, cert, cert_key, cacert)
        @url_path = url_path
        super(cert, cert_key, cacert)
        FileUtils.touch(state_file) if not File.exists?(state_file)
        @state = load_state || {}
        # 送信失敗時の再送上限
        @retry_limit = Dbbackup.conf['retry_limit'] || 10
      end

      def run
        if state[:state] == "FAILED"
          return resend
        else
          return send_file(create_backup_file)
        end
      ensure
        dump_state
      end

      def load_state
       return LTSV.load(state_file).first
      end

      def dump_state
        return File.write(state_file, LTSV.dump(@state))
      end

      private

      # 指定したファイルをHTTP POSTで送信
      def send_file(file_path, header={})
        return false if file_path.nil?

        # gzip圧縮したファイルを送信
        if file_path =~ /\.gz\z/
          gzip_file_path = file_path
        else
          gzip_file_path = "#{file_path}.gz"
        end

        state[:backup_file_path] = file_path
        state[:filename] = File.basename(file_path)
        state[:time] = Time.now.to_s
        state[:sha1] = Digest::SHA1.file(file_path).hexdigest

        boundary = SecureRandom.hex[0..5]
        # @client.debug_dev=STDOUT

        res = nil
        File.open(gzip_file_path) do |io|
          postdata = {
            "file" => io,
          }
          h = header.merge(
            "content-type" => "multipart/form-data; boundary=#{boundary}",
            "HTTP-X-Backup-SHA1" => Digest::SHA1.file(gzip_file_path).hexdigest,
          )
          if Dbbackup.conf["dbbackup_api_key"]
            h.merge!("HTTP-X-Dbbackup-Api-Key" => Dbbackup.conf["dbbackup_api_key"])
          end
          res = @client.post(File.join(host, @url_path), postdata, h)
        end
        if HTTP::Status.successful?(res.status)
          Dbbackup.logger.info("send backup file success: file=<#{file_path}>, http_status=<#{res.status}>")
          state[:state] = "SUCCESS"
          return true
        else
          state[:state] = "FAILED"
          raise "send backup file failed: file=<#{file_path}>, http_response=<#{res.inspect}>"
        end
      rescue => ex
        state[:state] = "FAILED"
        Dbbackup.logger.error("send backup file failed")
        raise ex
      end

      def resend
        send_file(state[:backup_file_path])
      end
    end
  end
end
