module Ginbee
  module Dbbackup
    # WALバックアップを送信する
    class WALUploader < Uploader
      # stateファイル、古いWALファイルなどを破棄する
      def clear
        delete_old_walfile
        state.clear
      ensure
        dump_state
      end

      private
      def create_backup_file
        delete_old_walfile

        # 新しいWALファイルを指定
        target = target_wal_file

        # 更新すべきファイルなし
        return nil if not target

        backup_file = File.join(Dbbackup.conf['work_dir'], File.basename(target))
        FileUtils.cp(target, backup_file)
        cmd = "gzip -f -k #{backup_file}"
        o,e,s = Open3.capture3(cmd)
        Dbbackup.logger.debug("executed gzip: cmd=<#{cmd}>,out=<#{o}>,err=<#{e}>")
        if not s.success?
          raise "gzip command error: process_status=<#{s.inspect},stdout=<#{o}>,stderr=<#{e}>"
        end
        # 更新ファイルのタイムスタンプを記録
        state[:mtime] = File.stat(target).mtime.iso8601(10)
        return backup_file
      end

      # 前回送信したWALファイルを削除
      def delete_old_walfile
        if state[:filename]
          old = File.join(Dbbackup.conf['work_dir'], state[:filename])
          if File.exists?(old) || File.exists?("#{old}.gz")
            FileUtils.rm_f(old)
            FileUtils.rm_f("#{old}.gz")
            Dbbackup.logger.debug("delete old wal: file=<#{old}>")
          end
        end
      end

      def state_file
        return File.join(Dbbackup.conf['work_dir'], "wal_upload.state")
      end

      # アップロード対象のWALファイルを取得
      def target_wal_file
        target =
          Dir.glob(File.join(Dbbackup.conf['wal_dir'], '*')).sort_by{|f|
            # 更新日時順
            File.stat(f).mtime
          }.detect{|file|
            if File.file?(file)
              # 初回実行時は一番古いファイルを選択
              break file if state[:mtime].nil?
              # 前回実行時以降で更新があった一番古いファイルを選択
              File.stat(file).mtime > Time.parse(state[:mtime])
            end
          }
        return target
      end

      def wal_number(filename)
        return filename.match(/[\w]+/)[0]
      end
    end
  end
end